#!/usr/bin/env python
import re
import sys
from sys import stderr
from tempfile import mkdtemp
from os.path import join, abspath, isdir, basename, exists
from os import access, F_OK, R_OK, X_OK
import os
from subprocess import Popen, STDOUT, PIPE

EX = {
    'tar.bz2': ['tar', 'xf'],
    'tar.gz': ['tar', 'xf'],
    'tgz': ['tar', 'xf'],
    'tar': ['tar', 'xf'],
    'zip': ['unzip'],
    'rar': ['rar', 'x'],
    '7z': ['7z', 'x'],
}

def errmsg(s):
    stderr.write(s + '\n')

def fn_prefix(fn):
    """get a filename, and converts it to a string apt to be used as prefix
    for the directory we'll create to extract the package
    
    removes the extension (.tar.gz, etc.) and gets rid of all the "nasty"
    characters in the name (e.g., " () [] and so on).
    """
    pat = '\\.(%s)$' % '|'.join(s.replace('.', '\\.') for s in EX)
    s = re.sub(pat, "", basename(fn)) + '-'
    s = re.sub(r"[^a-zA-Z0-9_.-]", r"", s)
    return s

def program_exists(program):
    """checks for the given program in all the places specified by $PATH
    
    returns True if it exists anywhere in $PATH, False otherwise"""
    program = basename(program)
    def is_exe(fpath):
        return exists(fpath) and access(fpath, X_OK)
    for path in os.environ["PATH"].split(os.pathsep):
        if is_exe(join(path, program)):
            return True
    return False

def indent(s):
    return "\n".join('    %s' % line for line in s.split('\n'))

def main():
    u'''
    extracts a package, without overwriting anything

    prints the path the file was extracted to, in stdout
    '''
    if len(sys.argv) != 2:
        errmsg("Safe Package Extractor\n\nUsage:\n%s <file name>" % sys.argv[0])
        sys.exit(1)
    fn = abspath(sys.argv[1])
    base_fn = basename(fn)

    # check that given file exists
    if not access(fn, R_OK):
        errmsg(u"File does not exist: '%s'" % fn)
        sys.exit(1)

    # check if we know how to extract the file. It is done checking its extension. Magic number based detection
    # will be available in a future version, hopefully
    pat = '\\.(%s)$' % '|'.join(s.replace('.', '\\.') for s in EX)
    base_fn = fn_prefix(fn)
    m = re.search(pat, fn)
    if not m:
        errmsg(u"File type unknown: '%s'" % fn)
        sys.exit(1)
    cd = os.getcwd()
    try:
        ndir = mkdtemp(prefix=base_fn, dir=cd)
    except OSError as e:
        errmsg(u"Can't extract into %s: '%s'" % fn)
        errmsg(u"Error message: '%s'" % unicode(e))
        sys.exit(1)
    ext = m.group(1)
    ep = EX[ext]
    if not program_exists(ep[0]):
        errmsg(u"Extractor program not found in path: '%s'" % ep[0])
        sys.exit(1)
    p = Popen(ep + [fn], cwd=ndir, stdout=PIPE, stderr=STDOUT)
    p.wait()
    if p.returncode != 0:
        errmsg(u"File extraction failed:\n")
        errmsg(indent("%s '%s'" % (u" ".join(ep), fn)))
        errmsg(u"")
        errmsg(u"Extractor program said:")
        errmsg(u"")
        errmsg(indent(p.stdout.read()))
        try:
            os.rmdir(ndir)
        except OSError, e:
            errmsg(u"")
            errmsg("Also, it kind-of extracted something. Go check at what is now available in '%s'" % ndir)
        sys.exit(1)
    files = os.listdir(ndir)
    if len(files) != 1:
        print ndir
        sys.exit(0)

    f = files[0]
    if not access(join(cd, f), F_OK):
        pdir = cd
        target_f = join(cd, f)
        os.rename(join(ndir, f), target_f)
        os.rmdir(ndir)
    else:
        pdir = ndir
        target_f = join(ndir, f)
    if isdir(target_f):
        print target_f
    else:
        print pdir

if __name__ == '__main__':
    main()
